<?php

namespace app\admin\controller;

use app\common\controller\Backend;
use think\Db;

/**
 * 入职申请管理
 *
 * @icon fa fa-circle-o
 */
class FactoryUser extends Backend
{
    
    /**
     * FactoryUser模型对象
     * @var \app\admin\model\FactoryUser
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\FactoryUser;
        $this->view->assign("statusList", $this->model->getStatusList());
    }

    public function import()
    {
        parent::import();
    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */
    

    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with(['user','factory'])
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                $row->visible(['id','status','join_time','quit_time','createtime']);
                $row->visible(['user']);
				$row->getRelation('user')->visible(['nickname']);
				$row->visible(['factory']);
				$row->getRelation('factory')->visible(['factory_name']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }

    /**
     * 审核
     */
    public function examine($ids = null)
    {
        if($ids) {
            $row = $this->model->get($ids);
            if (!$row) {
                $this->error(__('No Results were found'));
            }
            $adminIds = $this->getDataLimitAdminIds();
            if (is_array($adminIds)) {
                if (!in_array($row[$this->dataLimitField], $adminIds)) {
                    $this->error(__('You have no permission'));
                }
            }
            $params = $this->request->param();
            if($params['status'] == '2' && $row['status'] == '1'){
                $this->error('该申请已通过审核，无法驳回');
            }
            $row->status = $params['status'];
            Db::startTrans();
            try {
                // 入职时间
                if($params['status'] == '1'){
                    $row->join_time = time();
                    \app\admin\model\User::where('id',$row['user_id'])->update(['is_work'=>'1','factory_id'=>$row['factory_id'],'join_time'=>time()]);
                }
                $result = $row->save();
                Db::commit();
            } catch (ValidateException $e) {
                Db::rollback();
                $this->error($e->getMessage());
            } catch (PDOException $e) {
                Db::rollback();
                $this->error($e->getMessage());
            } catch (\think\Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if(!$result) {
                $this->error('审核失败');
            }
            $this->success('审核成功');
        }
    }

}
