<?php

namespace app\admin\controller\user;

use app\admin\model\Apply;
use app\admin\model\Deposit;
use app\admin\model\FactoryAdmin;
use app\admin\model\FactoryUser;
use app\admin\model\user\Realname;
use app\admin\model\UserSalary;
use app\api\model\JobCollect;
use app\api\model\JobKeyword;
use app\api\model\UserBorrow;
use app\api\model\UserKeyword;
use app\api\model\UserRecruitSubsidyLog;
use app\api\model\UserWorkLog;
use app\api\model\UserWorkSubsidyLog;
use app\common\controller\Backend;
use app\common\library\Auth;
use app\common\model\Record;
use Exception;
use fast\Tree;
use think\Db;
use think\exception\PDOException;

/**
 * 会员管理
 *
 * @icon fa fa-user
 */
class User extends Backend
{

    protected $relationSearch = true;
    protected $searchFields = 'id,username,nickname';

    /**
     * @var \app\admin\model\User
     */
    protected $model = null;
    protected $userlist = [];

    public function _initialize()
    {
        parent::_initialize();
        $this->model = model('User');

        $tree = Tree::instance();
        $tree->init(collection($this->model->with(['factory'])->order('id desc')->select())->toArray(), 'pid');
        $this->userlist = $tree->getTreeList($tree->getTreeArray(0), 'nickname');
        $userdata = [0 => __('None')];
        foreach ($this->userlist as $k => &$v) {
            $userdata[$v['id']] = $v['nickname'];
        }
        unset($v);
        $this->view->assign("userdata", $userdata);
        $this->view->assign("roledataList", $this->model->getRoledataList());

    }

    /**
     * 查看
     */
    public function index()
    {
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField'))
            {
                return $this->selectpage();
            }
            $filter = $this->request->request("filter");
            $filter = json_decode($filter,true);

            if(empty($filter)){
                $list = $this->userlist;
            }else{
                //构造父类select列表选项数据
                $list = [];

                foreach ($this->userlist as $k => $v) {
                    $nickname = true;
                    $mobile = true;
                    $is_work = true;
                    $is_complete = true;
                    $factory_shortname = true;
                    if(isset($filter['nickname']) && $filter['nickname']){
                        $nickname = stripos($v['nickname'], $filter['nickname']) !== false ? true : false;
                    }
                    if(isset($filter['mobile']) && $filter['mobile']){
                        $mobile = stripos($v['mobile'], $filter['mobile']) !== false ? true : false;
                    }
                    if(isset($filter['is_work']) && $filter['is_work'] != ''){
                        $is_work = $v['is_work'] == $filter['is_work'];
                    }
                    if(isset($filter['is_complete']) && $filter['is_complete'] != ''){
                        $is_complete = $v['is_complete'] == $filter['is_complete'];
                    }
                    if(isset($filter['factory.factory_shortname']) && $filter['factory.factory_shortname'] != ''){
                        $factory_shortname = stripos($v['factory']['factory_shortname'], $filter['factory.factory_shortname']) !== false ? true : false;
                    }
                    if ($nickname && $mobile && $is_work && $is_complete && $factory_shortname) {
                        $list[] = $v;
                    }
                }
            }

            $total = count($list);
            $result = array("total" => $total, "rows" => $list);

            return json($result);
        }
        return $this->view->fetch();
    }

    /**
     * 添加
     */
    public function add()
    {
        if ($this->request->isPost()) {
            $this->token();
        }
        return parent::add();
    }

    /**
     * 编辑
     */
    public function edit($ids = null)
    {
        if ($this->request->isPost()) {
            $this->token();
        }
        $row = $this->model->get($ids);
        $this->modelValidate = true;
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $this->view->assign('groupList', build_select('row[group_id]', \app\admin\model\UserGroup::column('id,name'), $row['group_id'], ['class' => 'form-control selectpicker']));
        $this->view->assign('start_time',date('Y-m-d',strtotime('-1month')));
        $this->view->assign('end_time',date('Y-m-d',strtotime('0day')));
        return parent::edit($ids);
    }

    /**
     * 删除
     */
    public function del($ids = "")
    {
        if (!$this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ? $ids : $this->request->post("ids");
        $row = $this->model->get($ids);
        $this->modelValidate = true;
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        Db::startTrans();
        try {
            Auth::instance()->delete($row['id']);
            // 职位报名
            Apply::where('user_id',$row['id'])->delete();
            // 提现
            Deposit::where('user_id',$row['id'])->delete();
            // 工厂管理
            FactoryAdmin::where('user_id',$row['id'])->delete();
            // 入职申请
            FactoryUser::where('user_id',$row['id'])->delete();
            // 职位收藏
            JobCollect::where('user_id',$row['id'])->delete();
            // 职位搜索关键字
            JobKeyword::where('user_id',$row['id'])->delete();
            // 实名认证
            Realname::where('user_id',$row['id'])->delete();
            // 通知记录
            Record::where('user_id',$row['id'])->delete();
            // 借支
            UserBorrow::where('user_id',$row['id'])->delete();
            // 搜索关键字
            UserKeyword::where('user_id',$row['id'])->delete();
            // 招聘补贴记录
            UserRecruitSubsidyLog::where('user_id',$row['id'])->delete();
            // 每月工资
            UserSalary::where('user_id',$row['id'])->delete();
            // 工时记录
            UserWorkLog::where('user_id',$row['id'])->delete();
            // 工时补贴记录
            UserWorkSubsidyLog::where('user_id',$row['id'])->delete();
            Db::commit();
        } catch (PDOException $e) {
            Db::rollback();
            $this->error($e->getMessage());
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        $this->success();
    }

}
